//+------------------------------------------------------------------+
//|                                                Angulo de MAs.mq5 |
//|                                        MetaQuotes Software Corp. |
//|                                                 http://dmffx.com |
//+------------------------------------------------------------------+

#include <include MovingAverages.mqh>

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_plots   1

#property indicator_type1   DRAW_LINE
#property indicator_color1  clrTurquoise       //clrDeepPink, clrDodgerBlue, clrLightGreen
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

#property indicator_label1  "MaAng"

#property indicator_level1       0
#property indicator_level2       15
#property indicator_level3       -15
#property indicator_level4        30
#property indicator_level5       -30

input mis_MA_METODOS       MAmetodo    =   EMA;  //Mtodo MA
input int                  MAPeriodo   =   21;
input ENUM_APPLIED_PRICE   MAPrecio    =   PRICE_TYPICAL;
input int                  velaSalto   =   2;
//input string               simbMA      =   "";
              //Mtodo MA: enum mis_MA_METODOS {SMA, EMA, SMMA, LWMA, DEMA, TEMA, LRMA, HMA, FRAMA, VIDYA, AMA, J2JMA};
              
int JMAperiodSuav       =   10;
int JMAfase             =   100;
int AMAFast             =   5;
int AMASlow             =   14;
int CMOPeriod           =   9;
int maxBarras     = -1;         //aqu se puede indicar el mximo de barras a calcular

int puntMA,
ENUM_TIMEFRAMES marcoTrab= PERIOD_CURRENT;
//--- indicator buffers
double AngBuffer[], MABuffer[];
string simbTrab;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   string etiqu= "MAAng("+etiquetaMA(MAmetodo, MAPeriodo, MAPrecio)+"-"+IaS(velaSalto)+") ";      //+abrevPrecio(MAPrecio);
//   if(simbMA=="") simbTrab= _Symbol;
   simbTrab= Symbol();
   SetIndexBuffer(0, AngBuffer, INDICATOR_DATA);
   SetIndexBuffer(1, MABuffer, INDICATOR_CALCULATIONS);
   ArraySetAsSeries(AngBuffer, true);
	ArraySetAsSeries(MABuffer, true);
   PlotIndexSetString(0,PLOT_LABEL,etiqu);
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
   IndicatorSetString(INDICATOR_SHORTNAME, etiqu);

   switch(MAmetodo)
     {
      case SMA:  puntMA= iMA(simbTrab,marcoTrab,MAPeriodo,0,MODE_SMA,MAPrecio); break;
      case EMA:  puntMA= iMA(simbTrab,marcoTrab,MAPeriodo,0,MODE_EMA,MAPrecio); break;
      case SMMA: puntMA= iMA(simbTrab,marcoTrab,MAPeriodo,0,MODE_SMMA,MAPrecio); break;
      case LWMA: puntMA= iMA(simbTrab,marcoTrab,MAPeriodo,0,MODE_LWMA,MAPrecio); break;
      case AMA:  puntMA= iAMA(simbTrab,marcoTrab,MAPeriodo,AMAFast,AMASlow,0,MAPrecio); break;
      case DEMA: puntMA= iDEMA(simbTrab,marcoTrab,MAPeriodo,0,MAPrecio); break;
      case FRAMA:puntMA= iFrAMA(simbTrab,marcoTrab,MAPeriodo,0,MAPrecio); break;
      case TEMA: puntMA= iTEMA(simbTrab,marcoTrab,MAPeriodo,0,MAPrecio); break;
      case VIDYA:puntMA= iVIDyA(simbTrab,marcoTrab,CMOPeriod,MAPeriodo,0,MAPrecio); break;
      case LRMA: puntMA= iCustom(simbTrab,marcoTrab,"LRMA",MAPeriodo,MAPrecio); break;
      case HMA:  puntMA= iCustom(simbTrab,marcoTrab,"HMA",MAPeriodo,MAPrecio); break;
      case JMA:  puntMA= iCustom(simbTrab,marcoTrab,"JMA",JMAperiodSuav,JMAfase,MAPrecio); break;
     }
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rangoTotal,
                const int preCalculado,
                const int begin,            // where the significant data start from
                const double& price[])       // array to calculate
{
   int k, inicio, limite, nCopiados, nBuffer= 0,
       contBarras= preCalculado;
   
   if (contBarras < 0) return(-1);
   else if (contBarras > 0) contBarras--;
   if(contBarras==0)inicio= 0;
   else inicio= contBarras;
   limite = rangoTotal - contBarras - velaSalto;         // - (MAPeriodo+velaSalto);
   if (maxBarras>0) limite = MathMin(limite, maxBarras);

   nCopiados= CopyBuffer(puntMA, nBuffer, 0, rangoTotal-inicio, MABuffer);
   if(nCopiados>0)
   {
      limite= MathMin(limite, nCopiados);
      for(k=inicio; k<limite; k++) AngBuffer[k]= anguloMA(MABuffer, k, marcoTrab, velaSalto, .9, simbTrab);
   }
   return(rangoTotal);
}

//----------------------------- ANGULO DE MEDIA MOVIL ---------------------------------------------
double anguloMA(double &arrayValores[], int indVela, ENUM_TIMEFRAMES marco, int barraSalto= 2, double factor_visual= 0.80, string simb=NULL)
{                     //DEVUELVE EL NGULO FORMADO POR LA LINEA DE UNA MEDIA MVIL, expresado en grados
	double MAactual= 0, MAprevia= 0;  //medias calculadas para la barra 0 y la barra 2. La tangente es (incremento en puntos / minutos transcurridos)
	double incremento= 0, tangente=0, angulo=0;
	//if(simb=="") simb= Symbol();
   MAactual= arrayValores[indVela];
	MAprevia= arrayValores[indVela+barraSalto];
	incremento= (MAactual - MAprevia) / valorPunto(simb);    //Point()
	tangente= incremento / (barraSalto*PeriodSeconds(marco)/60);
	angulo= MathArctan(tangente) / M_PI * 180 * factor_visual;         //ngulo en grados sexagesimales; si multiplicamos por 3200, milesimales.
	return(angulo);
}
